/*
 *    HHSnapIn.h  --  Header definition file for the HamHub Snap-In Interface
 *
 *    Copyright (C) 2004 Timewave Technology, Inc.
 *
 *    HH Snap-In DLL files are free software; you can redistribute them
 *    under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    The HH DLL snap-in interface is distributed WITHOUT ANY WARRANTY; 
 *    without even the implied warranty of MERCHANTABILITY or 
 *    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
 *    for more details.
 *
 *    For a copy of the GNU General Public License write to:
 *       Free Software Foundation, Inc.
 *       59 Temple Place, Suite 330
 *       Boston, MA  02111-1307  USA
 *
 *
 *    For API Programming information please see the file HHSnapIn.PDF.  This file
 *    is either included in this package or available for download.
 */

////////////////////////////////////////////////////////////////////////////////
// 
// Ham Hub Snapin DLL
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _HHSNAPIN_DLL
#define _HHSNAPIN_DLL

extern "C"
{

 // Predefined Mode definitions
#define	HH_PSK			0x00000001
#define	HH_MT63			0x00000002
#define	HH_MFSK			0x00000004
#define	HH_THROB		0x00000008
#define	HH_CW			0x00000010
#define	HH_RTTY			0x00000020
#define	HH_ASCII		0x00000040
#define	HH_FEC_AMTOR	0x00000080
#define	HH_AMTOR		0x00000100
#define	HH_FEC_PACTOR	0x00000200
#define	HH_PACTOR		0x00000400
#define	HH_FEC_GTOR		0x00000800
#define	HH_GTOR			0x00001000
#define	HH_PACTOR2		0x00002000
#define	HH_PACTOR3		0x00004000
#define	HH_CLOVER		0x00008000
#define	HH_NAVREX		0x00010000
#define	HH_SSTV			0x00020000
#define	HH_DUMB			0x00040000
#define	HH_USER			0x00080000
#define	HH_AFSK			0x00100000

 // Set the mode bit mask size
#ifndef HHMODE
#define	HHMODE	ULONGLONG
#endif //HHMODE

#ifndef _HH_HANDLE_SET
#define _HH_HANDLE_SET
typedef LPVOID	HH_HANDLE;
#endif // _HH_HANDLE_SET

 // Setup structure passed by the DLL discovery
typedef struct _tagSnapinSetup
{
	int		m_cbSize;
	WCHAR	m_wszName[256];
	HHMODE	m_dwMode;
	BOOL	m_bSelfDriven;
	BOOL	m_bDisableXMIT;
	BOOL	m_bDisableRECV;
} SNAPINSETUP;

//typedef BOOL (WINAPI DECODEHHDATA)(double *fData, int cbLen, LPSTR lpStr, int cbStr);

 ////////////////////////////////////////////////
 // Snap-In Functions
 ////////////////////////////////////////////////

 // Get the auto-discovery information
BOOL WINAPI GetHHSetup(SNAPINSETUP *);
 // Initialize - called when DLL is loaded
BOOL WINAPI InitHHMode(double fFrequency);
 // Get diddle/twiddle information
int WINAPI GetHHTwiddle(short *psData, int cbLen);
 // Encode the text string - return the soundcard information (8KHz, 16bit)
int WINAPI EncodeHHDataA(short *psData, int cbLen, LPSTR lpStr, int cbStr);
int WINAPI EncodeHHDataW(short *psData, int cbLen, LPWSTR lpStr, int cbStr);
 // Decode to a text string
int WINAPI DecodeHHDataA(short *psData, int cbLen, LPSTR lpStr, int cbStr);
int WINAPI DecodeHHDataW(short *psData, int cbLen, LPWSTR lpStr, int cbStr);
 // Invoke the setup dialog
UINT WINAPI InvokeHHSetup(HWND hWndParent);
 // Get the frequency passed in InitHHMode()
double WINAPI GetHHInternalFrequency(void);
 // Get the user toolbar window - attached to a CToolBar
HWND WINAPI GetHHToolbar(HWND hWndParent, DWORD dwTBStyle, UINT wID);
 // Invoke a user command
BOOL WINAPI DoHHCommand(UINT command);
 // Load and Save User information
BOOL WINAPI LoadHHUserData(LPVOID lpData, UINT cbData);
BOOL WINAPI SaveHHUserData(LPVOID lpData, UINT cbData);

 // HH COM Port Functions
 // These are for use with PK-Term ONLY!
 // They are not defined otherwise.

UINT WINAPI HHRequestComPort(void);
HH_HANDLE WINAPI HHOpenComPort(UINT nPort=0,				// Port Number or 0=Request Assignment
							   BYTE _cParity=NOPARITY,		// NOPARITY, EVENPARITY, ODDPARITY
							   BYTE _cStopBits=ONESTOPBIT,	// ONESTOPBIT, TWOSTOPBIT
							   BYTE _cByteSize=8,			// 5,7,8
							   DWORD _dwBaudRate=CBR_9600);	// See below
// Valid Baud Rates:
// CBR_110, CBR_300, CBR_600, CBR_1200, CBR_2400, CBR_4800, CBR_9600,
// CBR_14400, CBR_19200, CBR_38400, CBR_56000, CBR_57600, CBR_115200,
// CBR_128000, CBR_256000

void WINAPI HHCloseComPort(HH_HANDLE hPortHandle);
void WINAPI HHEnablePortRTS(HH_HANDLE hPortHandle, BOOL bAutomatic);
void WINAPI HHSetRTS(HH_HANDLE hPortHandle, BOOL bFlag);
void WINAPI HHWriteComPort(HH_HANDLE hPortHandle, LPBYTE lpData, UINT cbData);
UINT WINAPI HHReadComPort(HH_HANDLE hPortHandle, LPBYTE lpData, UINT cbData);

}; // extern "C"

#endif // _HHSNAPIN_DLL
