// HH_DUMB.cpp : Defines the initialization routines for the DLL.
//
// Copyright (c) 2004, Timewave Technology, Inc.
//
// Released under the GNU license.
//

#include "stdafx.h"
#include "resource.h"
#include <afxdllx.h>
#include <HHSnapIn.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

 // Handle to the COM port
HH_HANDLE	g_hComPort = NULL;
 // COM Port number
UINT g_nCOMPort;

 // Internal DLL data
static AFX_EXTENSION_MODULE HH_DUMBDLL = { NULL, NULL };

 // DLL entry point
extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("HH_DUMB.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(HH_DUMBDLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		new CDynLinkLibrary(HH_DUMBDLL);

	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("HH_DUMB.DLL Terminating!\n");

		 // TODO: Implement any destructor here
		if( g_hComPort )
			HHCloseComPort(g_hComPort);
		// Terminate the library before destructors are called
		AfxTermExtensionModule(HH_DUMBDLL);
	}
	return 1;   // ok
}

 // Get the setup information for the Snap-In.  This is used to generate the list of Snap-Ins
extern "C"
BOOL WINAPI GetHHSetup(SNAPINSETUP *pSetup)
{
	if( pSetup == NULL || pSetup->m_cbSize < sizeof(SNAPINSETUP) ) 
		return FALSE;

	 // Note: These must be wide/Unicode characters!
	wcscpy(pSetup->m_wszName, L"DUMB");
	 // The Mode.  See HHSnapIn for a list of modes
	pSetup->m_dwMode = HH_DUMB;
	 // Self Driven if there is no soundcard required.
	 // If set to FALSE, then all I/O is redirected out the soundcard
	pSetup->m_bSelfDriven = TRUE;

	return TRUE;
}

 // This is called when the DLL is started.
extern "C"
BOOL WINAPI InitHHMode(double fFrequency)
{
	 // Open the COM port
	 // TODO: Replace this with your initialization data
	g_nCOMPort = ::GetPrivateProfileInt(_T("Setup"), _T("ComPort"), 0, _T("HH_DUMB"));
	if( !g_nCOMPort )
		g_nCOMPort = HHRequestComPort();
	g_hComPort = HHOpenComPort(g_nCOMPort);
	return TRUE;
}

 // Return the set frequency
double WINAPI GetHHInternalFrequency(void)
{
	return (double)0;
}

 // Encode the data
extern "C"
int WINAPI EncodeHHDataW(short *fData, int cbLen, LPWSTR lpwStr, int cbStr)
{
	 // Send the data to the COM port
	 // TODO: Replace this with your code to create the waveform
	if( g_hComPort )
	{
		LPSTR lpStr = new CHAR[cbStr*2];
		cbStr = (int)wcstombs(lpStr, lpwStr, cbStr*2);
		HHWriteComPort(g_hComPort, (LPBYTE)lpStr, cbStr);
		delete [] lpStr;
	}
	return -1;
}

 // Encode the data into a waveform
extern "C"
int WINAPI EncodeHHDataA(short *fData, int cbLen, LPSTR lpStr, int cbStr)
{
	WCHAR *pwStr = new WCHAR[cbStr*2];
	cbStr = (int)mbstowcs(pwStr, lpStr, cbStr*2);
	int nLen = EncodeHHDataW(fData, cbLen, pwStr, cbStr);
	delete [] pwStr;
	return nLen;
}

 // Decode the data
extern "C"
int WINAPI DecodeHHDataW(short *fData, int cbLen, LPWSTR lpwStr, int cbStr)
{
	int nRead = 0;

	 // Get the data from the COM port
	 // TODO: Replace this with your code to decode the waveform
	if( g_hComPort )
	{
		LPSTR lpStr = new CHAR[cbStr];
		nRead = HHReadComPort(g_hComPort, (LPBYTE)lpStr, cbStr);
		nRead = (int)mbstowcs(lpwStr, lpStr, nRead);
		delete [] lpStr;
	}

	return nRead;
}

 // Convert from the waveform to the data
extern "C"
int WINAPI DecodeHHDataA(short *fData, int cbLen, LPSTR lpStr, int cbStr)
{
	WCHAR *pwStr = new WCHAR[cbStr*2];
	int nLen = DecodeHHDataW(fData, cbLen, pwStr, cbStr*2);
	nLen = (int)wcstombs(lpStr, pwStr, nLen);
	delete [] pwStr;
	return nLen;
}

 // Generate the Twiddle/Diddle
extern "C"
int WINAPI GetHHTwiddle(short *psData, int cbLen)
{
	 // No diddle for Dumb terminals
	 // TODO: Replace this with your diddle generator
	ZeroMemory(psData, cbLen*sizeof(short));

	return 0;
}

 // Simple dialog to change the COM port 
 // TODO: Replace the data with your parameters
BOOL CALLBACK SetupProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case WM_INITDIALOG:
			{
				TCHAR szData[64];
				_stprintf(szData, _T("%d"), g_nCOMPort);
				::SetDlgItemText(hDlg, IDC_EDIT_COMPORT, szData);
			}
			return TRUE;

		case WM_COMMAND:
			switch( LOWORD(wParam) )
			{
			case IDCANCEL:
				EndDialog(hDlg, IDCANCEL);
				return TRUE;
			case IDOK:
				{
					TCHAR	szData[64];

					::GetDlgItemText(hDlg, IDC_EDIT_COMPORT, szData, sizeof(szData)/sizeof(TCHAR));
					g_nCOMPort = _ttoi(szData);
				}
				EndDialog(hDlg, IDOK);
				return TRUE;
			default:
				return DoHHCommand(LOWORD(wParam));
			}
			break;

		case WM_CLOSE:
			DestroyWindow(hDlg);
			break;

		case WM_DESTROY:
			return TRUE;
	}
	return FALSE;
}

 // Pop up a setup dialog.
extern "C"
UINT WINAPI InvokeHHSetup(HWND hWndParent)
{
	 // Save the old port number
	UINT nOldPort = g_nCOMPort;

	 // Execute the dialog
	UINT nResult = (UINT)DialogBoxParam(HH_DUMBDLL.hResource, MAKEINTRESOURCE(IDD_DIALOG_SETUP), hWndParent, SetupProc, 0);

	 // Any changes?
	if( nResult == IDOK && nOldPort != g_nCOMPort )
	{
		 // Yes, so save the changes
		 // TODO: Replace this with your changes
		TCHAR szData[64];
		_stprintf(szData, _T("%d"), g_nCOMPort);
		::WritePrivateProfileString(_T("Setup"), _T("ComPort"), szData, _T("HH_DUMB"));
		if( g_hComPort )
			HHCloseComPort(g_hComPort);
		g_hComPort = 0;
		 // Reinitialize
		InitHHMode(0);
	}
	return nResult;
}

 // Create a toolbar
extern "C"
HWND WINAPI GetHHToolbar(HWND hWndParent, DWORD dwTBStyle, UINT wID)
{
	typedef struct 
	{
		WORD wVersion;
		WORD wWidth;
		WORD wHeight;
		WORD wItemCount;

		WORD* items()
			{ return (WORD*)(this+1); }
	} CToolBarData;

	 // Get the toolbar array
	HRSRC hRsrc = ::FindResource(HH_DUMBDLL.hResource, MAKEINTRESOURCE(IDR_TOOLBAR_CONFIG), RT_TOOLBAR);
	if (hRsrc != NULL)
	{
		HGLOBAL hGlobal = LoadResource(HH_DUMBDLL.hResource, hRsrc);
		if (hGlobal != NULL)
		{
			CToolBarData* pData = (CToolBarData*)LockResource(hGlobal);
			if (pData != NULL)
			{
				ASSERT(pData->wVersion == 1);

				 // Get the toolbar info
				SIZE sizeButtonBitmap, sizeButton;
				sizeButtonBitmap.cx = pData->wWidth;
				sizeButtonBitmap.cy = pData->wHeight;

				sizeButton = sizeButtonBitmap;
				sizeButton.cx += ::GetSystemMetrics(SM_CXEDGE);
				sizeButton.cy += ::GetSystemMetrics(SM_CYEDGE);

				int nResourceTotal = pData->wItemCount;

				 // Create a button list
				TBBUTTON	tbButton[2];

				tbButton[0].iBitmap = 0;
				tbButton[0].idCommand = pData->items()[0];
				tbButton[0].fsState = TBSTATE_ENABLED;
				tbButton[0].fsStyle = TBSTYLE_BUTTON;
				tbButton[0].iString = 0;

				 // Create the toolbar HWND
				HWND hWndToolbar = CreateToolbarEx(hWndParent, 
					dwTBStyle|WS_CHILD|WS_VISIBLE|WS_TABSTOP, 
					wID, 
					1, 
					HH_DUMBDLL.hResource, 
					IDR_TOOLBAR_CONFIG, 
					tbButton, 
					1, 
					sizeButton.cx, sizeButton.cy, 
					sizeButtonBitmap.cx, sizeButtonBitmap.cy, 
					sizeof(TBBUTTON));

				return hWndToolbar;
			}
		}
	}
	 // No Toolbar
	return NULL;
}

 // Commands from the returned toolbar
extern "C"
BOOL WINAPI DoHHCommand(UINT command)
{
	if( command == ID_TOOLBAR_RESET )
	{
		if( g_hComPort )
			HHCloseComPort(g_hComPort);
		InitHHMode(0);
		return TRUE;
	}
	return FALSE;
}


 // Load data
extern "C"
BOOL WINAPI LoadHHUserData(LPVOID lpData, UINT cbData)
{
	return ::GetPrivateProfileStruct(_T("Setup"), _T("UserData"), lpData, cbData, _T("HH_DUMB"));
}

// Save data
extern "C"
BOOL WINAPI SaveHHUserData(LPVOID lpData, UINT cbData)
{
	return ::WritePrivateProfileStruct(_T("Setup"), _T("UserData"), lpData, cbData, _T("HH_DUMB"));
}
