// HH_MT63.cpp : Defines the initialization routines for the DLL.
//
// DLL Front End  Copyright (c) 2004 Timewave Technology Inc.
//    Written by ehg 9/2004
//    Released under the GNU license
//
// Other Portions Copyright (c) 1999-2004 Pawel Jalocha, SP9VRC
//    Changes made to mt63 code: (ehg)
//       1. Moved RX DataInterleave var to public
//    Released under the GNU license
//

#include "stdafx.h"
#include <afxdllx.h>
#include <HHSnapIn.h>
#include "dsp.h"
#include "mt63.h"
#include <mmsystem.h>
#include "Setup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// ============================================================================

double g_fReqRate=8000.0;	// we request 8000 Hz sampling

int Bandwidth=1000;		// 500, 1000 or 2000 Hz bandwidth
int Interleave=0;		// short/long interleave
int IntegLen=32;        // integration period for sync./data tracking

bool g_bInterleaveTX=false;
int g_nEndTX = 0;
bool g_bInterleaveRX=false;
int g_nEndRX = 0;
int g_nStartRX = 0;
bool g_bLastTX=true;
float g_fSNR = 9.5;
float g_fConf = 0.2;
float g_fAmplification = 2.0;

bool g_bFilterRXIntegration = true;

char *CW_ID=NULL;		// CW identification to be transmitted along the digital signal

// ============================================================================

RateConvBL RateConvRX;	// rate converter to adjust the sampling rate
BOOL UseRateConvRX=TRUE;
RateConvBL RateConvTX;	// rate converter to adjust the sampling rate
BOOL UseRateConvTX=TRUE;

float_buff InpFloat;

double g_fCorrection = 1.0075;

LevelMonitor InpLevel;

// ============================================================================

static MT63rx *pRX = NULL;
static MT63tx *pTX = NULL;

static AFX_EXTENSION_MODULE HH_MT63DLL = { NULL, NULL };

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	// Remove this if you use lpReserved
	UNREFERENCED_PARAMETER(lpReserved);

	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TRACE0("HH_MT63.DLL Initializing!\n");
		
		// Extension DLL one-time initialization
		if (!AfxInitExtensionModule(HH_MT63DLL, hInstance))
			return 0;

		// Insert this DLL into the resource chain
		// NOTE: If this Extension DLL is being implicitly linked to by
		//  an MFC Regular DLL (such as an ActiveX Control)
		//  instead of an MFC application, then you will want to
		//  remove this line from DllMain and put it in a separate
		//  function exported from this Extension DLL.  The Regular DLL
		//  that uses this Extension DLL should then explicitly call that
		//  function to initialize this Extension DLL.  Otherwise,
		//  the CDynLinkLibrary object will not be attached to the
		//  Regular DLL's resource chain, and serious problems will
		//  result.

		new CDynLinkLibrary(HH_MT63DLL);

		Bandwidth = ::GetPrivateProfileInt(_T("Setup"), _T("Bandwidth"), Bandwidth, _T("HH_MT63"));
		if( Bandwidth != 500 && Bandwidth != 1000 && Bandwidth != 2000 ) Bandwidth = 1000;
		Interleave = ::GetPrivateProfileInt(_T("Setup"), _T("Interleave"), Interleave, _T("HH_MT63"));
		if( Interleave > 1 || Interleave < 0 ) Interleave = 0;
		IntegLen = ::GetPrivateProfileInt(_T("Setup"), _T("IntegLen"), IntegLen, _T("HH_MT63"));
		if( IntegLen < 32 || IntegLen > 64 ) IntegLen = 32;
		g_fCorrection = ::GetPrivateProfileInt(_T("Setup"), _T("Correction"), (int)(g_fCorrection*10000.0), _T("HH_MT63"))/10000.0;
		if( g_fCorrection < 0.9 || g_fCorrection > 1.1 ) g_fCorrection = 1.0075;
		g_fSNR = ::GetPrivateProfileInt(_T("Setup"), _T("SNR"), (g_fSNR*100), _T("HH_MT63"))/100.0;
		if( g_fSNR < 2 || g_fSNR > 24 ) g_fSNR = 9.5;
		g_fConf = ::GetPrivateProfileInt(_T("Setup"), _T("Conf"), (g_fConf*100), _T("HH_MT63"))/100.0;
		if( g_fConf < 0.1 || g_fConf > 0.5 ) g_fConf = 0.2;
		g_fAmplification = ::GetPrivateProfileInt(_T("Setup"), _T("Amp"), (g_fAmplification*1000), _T("HH_MT63"))/1000.0;
		if( g_fAmplification < 0.2 || g_fAmplification > 20 ) g_fAmplification = 2;
		g_bFilterRXIntegration = ::GetPrivateProfileInt(_T("Setup"), _T("FilterRXInteg"), FALSE, _T("HH_MT63"))==TRUE;

		pRX = new MT63rx;
		pTX = new MT63tx;
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		TRACE0("HH_MT63.DLL Terminating!\n");

		if( pRX )
			delete pRX;
		if( pTX )
			delete pTX;

		// Terminate the library before destructors are called
		AfxTermExtensionModule(HH_MT63DLL);
	}
	return 1;   // ok
}

 // Return the snap-in configuration
extern "C"
BOOL WINAPI GetHHSetup(SNAPINSETUP *pSetup)
{
	if( pSetup == NULL || pSetup->m_cbSize != sizeof(SNAPINSETUP) ) 
		return FALSE;

	wcscpy(pSetup->m_wszName, L"MT63");
	pSetup->m_dwMode = HH_MT63;
	pSetup->m_bSelfDriven = FALSE;

	return TRUE;
}

 // Initialize the snap-in
extern "C"
BOOL WINAPI InitHHMode(double fFrequency)
{
	if( !pRX )
	{
		TRACE(_T("MT63: InitHHMode: pRX not allocated\n"));
		return FALSE;
	}

	if( !pTX )
	{
		TRACE(_T("MT63: InitHHMode: pTX not allocated\n"));
		return FALSE;
	}

	TRACE(_T("MT63: InitHHMode\n"));

	if( fFrequency < 6000 )
	{
		TRACE(_T("MT63: InitHHMode: Invalid initialization frequency!\n"));
		fFrequency = 8000;
	}

	g_fReqRate = fFrequency;

	int err;

	 ////////////////////////////////////////////
	 // RX

	TRACE(_T("InitHHMode:RX: Bandwidth=%ld, Interleave=%ld, IntegLen=%ld\n"), (long)Bandwidth, (long)Interleave, (long)IntegLen);

	InpLevel.Preset(64.0, 0.75);

	 // Adjust the frequency for soundcard drift
	fFrequency = g_fReqRate/g_fCorrection;
	UseRateConvRX = (fFrequency!=8000.0)?TRUE:FALSE;

	if( UseRateConvRX )
	{ 
		err=RateConvRX.Preset(32,NULL,16);
		if(err) 
		{
			TRACE(_T("MT63: RateConv RX Preset: Not Enough RAM!\n"));
			return FALSE;
		}
		err=RateConvRX.ComputeShape(0.0,(float)M_PI,WindowHamming);
		if(err) 
		{
			TRACE(_T("MT63: RateConv RX Shape: Not Enough RAM!\n"));
			return FALSE;
		}
		RateConvRX.SetOutVsInp((float)(8000.0/fFrequency));
	}
	err=pRX->Preset(Bandwidth,Interleave,IntegLen);
	if(err) 
	{
		TRACE(_T("MT63: Preset RX: Unable to initialize.\n"));
		return FALSE;
	}

	 ////////////////////////////////////////////
	 // TX

	TRACE(_T("InitHHMode:TX: Bandwidth=%ld, Interleave=%ld, CW="), (long)Bandwidth, (long)Interleave);
#ifdef _UNICODE
	TRACE(L"%S\n", CW_ID);
#else
	TRACE("%s\n", CW_ID);
#endif
	 // Adjust the frequency for soundcard drift
	fFrequency = g_fReqRate*g_fCorrection;
	UseRateConvTX = (fFrequency!=8000.0)?TRUE:FALSE;

	if( UseRateConvTX )
	{ 
		err=RateConvTX.Preset(32,NULL,16);
		if(err) 
		{
			TRACE(_T("MT63: RateConv TX Preset: Not Enough RAM!\n"));
			return FALSE;
		}
		err=RateConvTX.ComputeShape(0.0,(float)M_PI,WindowHamming);
		if(err) 
		{
			TRACE(_T("MT63: RateConv TX Shape: Not Enough RAM!\n"));
			return FALSE;
		}
		RateConvTX.SetOutVsInp((float)(fFrequency/8000.0));
	}
	err=pTX->Preset(Bandwidth,Interleave,CW_ID);
	if(err) 
	{
		TRACE(_T("MT63: Preset TX: Unable to initialize.\n"));
		return FALSE;
	}

	g_bInterleaveTX=false;
	g_nEndTX = 0;
	g_bInterleaveRX=false;
	g_nEndRX = 0;
	return TRUE;
}

extern "C"
int WINAPI EncodeHHDataA(short *psData, int cbLen, LPSTR lpStr, int cbStr)
{
	g_bLastTX = true;

	if( !pTX ) return 0;

	s16_buff OutBuff;

	 // Loop through and send each char
	int nOffset = 0;
	for( int i = 0; i < cbStr && nOffset < cbLen; ++i )
	{
		 // Send the char
		pTX->SendChar(lpStr[i]);
		 // Convert it to the waveform
		float *pData = NULL;
		int nData = 0;
		if( UseRateConvTX )
		{ 
			RateConvTX.Process(&pTX->Comb.Output);
			pData = RateConvTX.Output.Data;
			nData = RateConvTX.Output.Len;
		} 
		else
		{
			pData = pTX->Comb.Output.Data;
			nData = pTX->Comb.Output.Len;
		}

		 // Convert it to standard 16 bit
		int nCopy = nData;
		if( nOffset+nData > cbLen )
			nCopy = cbLen-nOffset;
		 // Append it to the returned array
		ConvFloatToS16(pData, &psData[nOffset], nCopy, 32768.0f);
		nOffset += nCopy;
	}
	g_bInterleaveTX = true;
	g_nEndTX = pTX->DataInterleave+pTX->DataInterleave/2;

	return nOffset;
}

extern "C"
int WINAPI DecodeHHDataA(short *psData, int cbLen, LPSTR lpStr, int cbStr)
{
	 // Reduce the garbage a bit
	if( g_bLastTX && g_bFilterRXIntegration )
		g_nStartRX = pRX->DataInterleave;

	g_bLastTX = false;

	if( !pRX ) return 0;

	int nLen = 0;

	if( g_fAmplification == 0 )
		g_fAmplification = 1;

	 // Convert it to floating point (0.0 - 1.0)
	if( psData )
	{
		//InpFloat.EnsureSpace(cbLen);
		ConvS16toFloat(psData, &InpFloat, cbLen, 1.0f/(32768.0f/g_fAmplification));
		// Check the input level
		InpLevel.Process(&InpFloat);
	}
	else
	{
		 // Insert dummy data for pushing out the interleave data
		InpFloat.EnsureSpace(8000);
		memset(InpFloat.Data, 0x00, sizeof(InpFloat.Data[0])*8000);
		InpFloat.Len = 8000;
	}

	 // Convert it
	if(UseRateConvRX)
	{ 
		RateConvRX.ProcessLinI(&InpFloat);
		pRX->Process(&RateConvRX.Output);
	} 
	else 
	{
		pRX->Process(&InpFloat);
	}

	 // Copy the output
	float SNR=pRX->FEC_SNR(); if(SNR>99.9) SNR=99.9;
	float fConf = pRX->SYNC_Confidence();

	TRACE(_T("SNR = %6.2f, Conf = %.2f, "), (double)SNR, (double)fConf);

	if( psData && SNR >= g_fSNR && fConf >= g_fConf )
	{
		TRACE(_T("Accepted.\n"));
		nLen = min(pRX->Output.Len, cbStr);
		int idx = 0;
		for(int i=0; i < nLen; i++, g_nStartRX--)
		{ 
			if( g_nStartRX <= 0 )
			{
				BYTE code = pRX->Output.Data[i]; 
				lpStr[idx++] = code;
			}
		}
		nLen = idx;
		g_bInterleaveRX=true;
		g_nEndRX = pRX->DataInterleave+pRX->DataInterleave/2;
		if( g_nStartRX <= 0 )
		{
			g_nStartRX = 0;
		}
	}
	 // Make sure we clock in the rest of the data
	else if( g_bInterleaveRX || (!psData && g_nEndRX > 0) )
	{
		TRACE(_T("Clocked.\n"));
		nLen = min(pRX->Output.Len, cbStr);
		int idx = 0;
		for(int i=0; i < nLen; i++, g_nStartRX--, g_nEndRX--)
		{ 
			if( g_nEndRX > 0 && g_nStartRX <= 0 )
			{
				BYTE code = pRX->Output.Data[i]; 
				lpStr[idx++] = code;
			}
		}
		nLen = idx;
		if( g_nEndRX <= 0 )
		{
			g_nEndRX = 0;
			g_bInterleaveRX=false;
		}
	}
	 // Nothing left
	else
	{
		TRACE(_T("Invalid.\n"));
		g_nStartRX -= pRX->Output.Len;
		 // Set this here so that data smaller than the interleave gets clocked in
		g_nEndRX = pRX->DataInterleave+pRX->DataInterleave/2;
		g_bInterleaveRX=false;
		nLen = 0;
	}

	return nLen;
}

extern "C"
BOOL WINAPI HasHHData(void)
{
	if( g_bInterleaveTX ) return TRUE;
	return FALSE;
}

 // Get the diddle (sync up the interleave)
extern "C"
int WINAPI GetHHTwiddle(short *psData, int cbLen)
{
	int nOffset = 0;

	ZeroMemory(psData, cbLen*sizeof(short));

	 // Send an extra set of NULLs to clock out 
	 // the last of the data in the interleave
	if( g_bInterleaveTX )
	{
		int nCopy = 0;
		for(int i=0; nOffset+nCopy < cbLen && g_nEndTX-- > 0; i++)
		{
			pTX->SendChar('\0');
			float *pData = NULL;
			int nData = 0;
			if( UseRateConvTX )
			{ 
				RateConvTX.ProcessLinI(&pTX->Comb.Output);
				pData = RateConvTX.Output.Data;
				nData = RateConvTX.Output.Len;
			} 
			else
			{
				pData = pTX->Comb.Output.Data;
				nData = pTX->Comb.Output.Len;
			}

			nCopy = nData;
			if( nOffset+nData > cbLen )
				nCopy = cbLen-nOffset;
			ConvFloatToS16(pData, &psData[nOffset], nCopy, 32768.0f);
			nOffset += nCopy;
		}
		if( g_nEndTX <= 0 )
		{
			g_bInterleaveTX = false;
			g_nEndTX = 0;
		}
	}
	else
	{
		int nCopy = 0;
		g_nEndTX = 0;
		// Send the Jam signal - note: this is very large
		for(int i=0; i<20 && nOffset+nCopy < cbLen; i++)
		{
			pTX->SendJam();
			float *pData = NULL;
			int nData = 0;
			if( UseRateConvTX )
			{ 
				RateConvTX.ProcessLinI(&pTX->Comb.Output);
				pData = RateConvTX.Output.Data;
				nData = RateConvTX.Output.Len;
			} 
			else
			{
				pData = pTX->Comb.Output.Data;
				nData = pTX->Comb.Output.Len;
			}

			nCopy = nData;
			if( nOffset+nData > cbLen )
				nCopy = cbLen-nOffset;
			ConvFloatToS16(pData, &psData[nOffset], nCopy, 32768.0f);
			nOffset += nCopy;
		}
	}

	return nOffset;
}

 // Show the setup dialog - self explanitory
UINT WINAPI InvokeHHSetup(HWND hWndParent)
{
	afxCurrentResourceHandle = HH_MT63DLL.hResource;
	afxCurrentInstanceHandle = HH_MT63DLL.hModule;

	CSetup	setup(CWnd::FromHandlePermanent(hWndParent));

	setup.m_bFilterRXInteg = g_bFilterRXIntegration;
	setup.m_fFSC = g_fReqRate * g_fCorrection;
	setup.m_radioInterleave = Interleave?1:0;
	setup.m_nIntegration = IntegLen;
	setup.m_fSNR = g_fSNR;
	setup.m_fConf = g_fConf;
	setup.m_fAmp = g_fAmplification;
	switch(Bandwidth)
	{
	case 500: setup.m_radioBandwidth = 0; break;
	case 1000: setup.m_radioBandwidth = 1; break;
	case 2000: setup.m_radioBandwidth = 2; break;
	default: setup.m_radioBandwidth = 1; break;
	}

	if( (UINT)setup.DoModal() == IDOK )
	{
		g_fCorrection = setup.m_fFSC / g_fReqRate;
		g_bFilterRXIntegration = setup.m_bFilterRXInteg;
		Interleave = setup.m_radioInterleave?1:0;
		IntegLen = setup.m_nIntegration;
		switch(setup.m_radioBandwidth)
		{
		case 0: Bandwidth = 500; break;
		case 1: Bandwidth = 1000; break;
		case 2: Bandwidth = 2000; break;
		default: Bandwidth = 1000; break;
		}
		g_fSNR = setup.m_fSNR;
		g_fConf = setup.m_fConf;
		g_fAmplification = setup.m_fAmp;

		TCHAR szOut[32];
		_stprintf(szOut, _T("%d"), Bandwidth); ::WritePrivateProfileString(_T("Setup"), _T("Bandwidth"), szOut, _T("HH_MT63"));
		_stprintf(szOut, _T("%d"), Interleave); ::WritePrivateProfileString(_T("Setup"), _T("Interleave"), szOut, _T("HH_MT63"));
		_stprintf(szOut, _T("%d"), IntegLen); ::WritePrivateProfileString(_T("Setup"), _T("IntegLen"), szOut, _T("HH_MT63"));
		_stprintf(szOut, _T("%d"), (int)(g_fCorrection*10000.0)); ::WritePrivateProfileString(_T("Setup"), _T("Correction"), szOut, _T("HH_MT63"));
		_stprintf(szOut, _T("%d"), (int)(g_fSNR*100.0)); ::WritePrivateProfileString(_T("Setup"), _T("SNR"), szOut, _T("HH_MT63"));
		_stprintf(szOut, _T("%d"), (int)(g_fConf*100.0)); ::WritePrivateProfileString(_T("Setup"), _T("Conf"), szOut, _T("HH_MT63"));
		_stprintf(szOut, _T("%d"), (int)(g_fAmplification*1000.0)); ::WritePrivateProfileString(_T("Setup"), _T("Amp"), szOut, _T("HH_MT63"));
		_stprintf(szOut, _T("%d"), g_bFilterRXIntegration?1:0); ::WritePrivateProfileString(_T("Setup"), _T("FilterRXInteg"), szOut, _T("HH_MT63"));
		if( !g_bFilterRXIntegration ) g_nStartRX = 0;

		::InitHHMode(g_fReqRate);
		return IDOK;
	}
	return IDCANCEL;
}

double WINAPI GetHHInternalFrequency(void)
{
	return (double)Bandwidth;
}
